/*******************************************************************************
 * File Name: CapSenseMyTuner.c
 *
 * Version: 1.00
 *
 * Description:
 *  This file contains BLE related functions.
 * 
 ********************************************************************************
 * Copyright 2019, Cypress Semiconductor Corporation.  All rights reserved.
 * You may use this file only in accordance with the license, terms, conditions,
 * disclaimers, and limitations in the end user license agreement accompanying
 * the software package with which this file was provided.
 *******************************************************************************/

#include "CapSenseMyTuner.h"
#include "project.h"

/* Tuner states definition */
#define TU_FSM_RUNNING      (0u)
#define TU_FSM_SUSPENDED    (1u)
#define TU_FSM_SNR_TEST     (2u)
#define TU_FSM_ONE_SCAN     (3u)

/* Tuner state variable */
static uint32 tunerFsm = TU_FSM_RUNNING;

/*******************************************************************************
* Function Name: CapSense_MyRunTuner
****************************************************************************//**
*
* \brief
*  Establishes synchronized communication with the Tuner application.
*
* \details
*  This function is used to establish synchronized communication between the
*  CapSense Component and Tuner application (or other host controllers).
*  This function is called periodically in the application program loop
*  to serve the Tuner application (or host controller) requests and commands.
*  In most cases, the best place to call this function is after processing and
*  before next scanning.
*
*  If this function is absent in the application program, then communication is
*  asynchronous and the following disadvantages are applicable:
*    - The raw counts displayed in the tuner may be filtered and/or unfiltered.
*      As a result, noise and SNR measurements will not be accurate.
*    - The Tuner tool may read the sensor data such as raw counts from a scan
*      multiple times, as a result, noise and SNR measurement will not be
*      accurate.
*    - The Tuner tool and host controller should not change the Component parameters
*      via the tuner interface. Changing the Component parameters via the tuner
*      interface in the async mode will result in Component abnormal behavior.
*
*  Note that calling this function is not mandatory for the application, but
*  required only to synchronize the communication with the host controller or
*  tuner application.
*
* \return
*  In some cases, the application program may need to know if the Component was
*  re-initialized. The return indicates if a restart command was executed or not:
*    - CapSense_STATUS_RESTART_DONE - Based on a received command, the
*      Component was restarted.
*    -  CapSense_STATUS_RESTART_NONE - No restart was executed by this
*      function.
*
*******************************************************************************/
uint32 CapSense_MyRunTuner(void)
{
    uint32 interruptState;
    uint32 widgetId;
    uint32 sensorId;
    uint32 updateFlag = 0Lu;
    CapSense_TU_CMD_ENUM command;
    uint32 tunerStatus = CapSense_STATUS_RESTART_NONE;

    volatile CapSense_RAM_STRUCT *ptrDsRam;
    volatile CapSense_RAM_SNS_STRUCT *ptrSnsTmp;

    ptrDsRam = &CapSense_dsRam;

    do
    {
        /* ONE_SCAN command could be interpreted as two commands:
        * RESUME till next call of this function and then
        * SUSPEND till next command receiving.
        */
        if (TU_FSM_ONE_SCAN == tunerFsm)
        {
            interruptState = CyEnterCriticalSection();
            ptrDsRam->tunerCmd = (uint16)CapSense_TU_CMD_SUSPEND_E;
           CyExitCriticalSection(interruptState);
        }
        command = (CapSense_TU_CMD_ENUM)ptrDsRam->tunerCmd;
        
        /* Send Data to the CapSense Tuner tool */
       TunerSend();
        
        /* Call user's callback function if it is registered */
       TunerReceive();

        /* Check command register */
        switch (command)
        {
        case CapSense_TU_CMD_SUSPEND_E:
            tunerFsm = TU_FSM_SUSPENDED;
            updateFlag = 1Lu;
            flag = 1u;
            break;

        case CapSense_TU_CMD_RESUME_E:
            tunerFsm = TU_FSM_RUNNING;
            updateFlag = 1Lu;
            break;

        case CapSense_TU_CMD_RESTART_E:
            tunerStatus = CapSense_STATUS_RESTART_DONE;
            tunerFsm = TU_FSM_RUNNING;
            updateFlag = 1Lu;
            break;

        case CapSense_TU_CMD_RUN_SNR_TEST_E:
            tunerFsm = TU_FSM_SNR_TEST;
            updateFlag = 1Lu;
            break;

        case CapSense_TU_CMD_PING_E:
            tunerFsm = TU_FSM_RUNNING;
            updateFlag = 1Lu;
            break;

        case CapSense_TU_CMD_ONE_SCAN_E:
            tunerFsm = TU_FSM_ONE_SCAN;
            updateFlag = 0Lu;
            flag = 0u;
            break;

        default:
            break;
        }

        /* Set Complete flag in command register if needed */
        if (0Lu != updateFlag)
        {
            interruptState =  CyEnterCriticalSection();

            /* Check if command wasn't overwritten with new command */
            if (command == (CapSense_TU_CMD_ENUM)ptrDsRam->tunerCmd)
            {
                ptrDsRam->tunerCmd |= CapSense_TU_CMD_COMPLETE_BIT;
            }
            CyExitCriticalSection(interruptState);

            updateFlag = 0Lu;
        }
    } while (TU_FSM_SUSPENDED == tunerFsm && flag == 0);

    /* Check for SNR Test mode and do appropriate actions */
    if (TU_FSM_SNR_TEST == tunerFsm)
    {
        widgetId = ptrDsRam->snrTestWidgetId;
        sensorId = ptrDsRam->snrTestSensorId;

        /* Check if widgetId and sensorId are in range,
         * and data was updated in RAM (scan counter changed)
         */
        if ((widgetId < CapSense_TOTAL_WIDGETS)              &&
            (sensorId < CapSense_GET_SENSOR_COUNT(widgetId)) &&
            (ptrDsRam->scanCounter != ptrDsRam->snrTestScanCounter))
        {
            /* Get access to the Sensor Object */
            ptrSnsTmp = CapSense_dsFlash.wdgtArray[widgetId].ptr2SnsRam;
            ptrSnsTmp += sensorId;

            /* Update data in SNR Test Object */
            interruptState =  CyEnterCriticalSection();
            ptrDsRam->snrTestScanCounter = ptrDsRam->scanCounter;
            ptrDsRam->snrTestRawCount[0u] = ptrSnsTmp->raw[0u];

            #if (0u != CapSense_MULTI_FREQ_SCAN_EN)
                ptrDsRam->snrTestRawCount[1u] = ptrSnsTmp->raw[1u];
                ptrDsRam->snrTestRawCount[2u] = ptrSnsTmp->raw[2u];
            #endif

           CyExitCriticalSection(interruptState);
        }
    }
    return tunerStatus;
}

/* [] END OF FILE */